@@ -65,7 +65,7 @@ struct BigCell {
     struct BigCell *prev;
 
     struct MapCellLayer head;
-    struct MapCellLayer tail;
+    struct MapCellTailLayer tail;
 
     guint16 x, y;
     guint8 layer;
@@ -76,10 +76,8 @@ static void recenter_virtual_map_view(int diff_x, int diff_y);
 static void mapdata_get_image_size(int face, guint8 *w, guint8 *h);
 
 
-/**
- * Viewable map size, in number of tiles.
- */
-static int width, height;
+static int width;   //< width of current map view
+static int height;  //< height of current map view
 
 
 /**
@@ -95,20 +93,19 @@ static struct BigCell *bigfaces_head;
  */
 static struct BigCell bigfaces[MAX_VIEW][MAX_VIEW][MAXLAYERS];
 
-
-struct Map the_map;
+static struct Map the_map;
 
 /**
  * Clear cells the_map.cells[x][y..y+len_y-1].
  */
 static void clear_cells(int x, int y, int len_y) {
     int clear_cells_i, j;
-    memset(&the_map.cells[(x)][(y)], 0, sizeof(the_map.cells[(x)][(y)])*(len_y));
+    memset(mapdata_cell(x, y), 0, sizeof(struct MapCell)*len_y);
 
     for (clear_cells_i = 0; clear_cells_i < (len_y); clear_cells_i++) {
         for (j=0; j < MAXLAYERS; j++) {
-            the_map.cells[(x)][(y)+clear_cells_i].heads[j].size_x = 1;
-            the_map.cells[(x)][(y)+clear_cells_i].heads[j].size_y = 1;
+            mapdata_cell(x, y+clear_cells_i)->heads[j].size_x = 1;
+            mapdata_cell(x, y+clear_cells_i)->heads[j].size_y = 1;
         }
     }
 }
@@ -116,15 +113,15 @@ static void clear_cells(int x, int y, int len_y) {
 /**
  * Get the stored map cell at the given map coordinate.
  */
-struct MapCell *mapdata_cell(int x, int y) {
-    return &the_map.cells[x][y];
+inline struct MapCell *mapdata_cell(int x, int y) {
+    return &the_map._cells[x][y];
 }
 
 /**
  * Determine whether the map data contains the given cell.
  */
 bool mapdata_contains(int x, int y) {
-    if (x < 0 || y < 0 || the_map.x <= x || the_map.y <= y) {
+    if (x < 0 || y < 0 || the_map.width <= x || the_map.height <= y) {
         return false;
     }
 
@@ -141,11 +138,11 @@ bool mapdata_can_smooth(int x, int y, int layer) {
  */
 void mapdata_size(int *x, int *y) {
     if (x != NULL) {
-        *x = the_map.x;
+        *x = the_map.width;
     }
 
     if (y != NULL) {
-        *y = the_map.y;
+        *y = the_map.height;
     }
 }
 
@@ -175,16 +172,16 @@ static void set_darkness(int x, int y, int darkness)
             && (use_config[CONFIG_LIGHTING] == CFG_LT_PIXEL
                 ||  use_config[CONFIG_LIGHTING] == CFG_LT_PIXEL_BEST)) {
         if (x > 1) {
-            the_map.cells[x-1][y].need_update = 1;
+            mapdata_cell(x-1, y)->need_update = 1;
         }
         if (y > 1) {
-            the_map.cells[x][y-1].need_update = 1;
+            mapdata_cell(x, y-1)->need_update = 1;
         }
         if (x < width-1) {
-            the_map.cells[x+1][y].need_update = 1;
+            mapdata_cell(x+1, y)->need_update = 1;
         }
         if (y < height-1) {
-            the_map.cells[x][y+1].need_update = 1;
+            mapdata_cell(x, y+1)->need_update = 1;
         }
     }
 }
@@ -196,9 +193,9 @@ static void mark_resmooth(int x, int y, int layer)
         for (sdx=-1; sdx<2; sdx++)
             for (sdy=-1; sdy<2; sdy++)
                 if ( (sdx || sdy) /* ignore (0,0) */
-                        &&  ( (x+sdx >0) && (x+sdx < FOG_MAP_SIZE) &&  /* only inside map */
-                              (y+sdy >0) && (y+sdy < FOG_MAP_SIZE) ) ) {
-                    the_map.cells[x+sdx][y+sdy].need_resmooth=1;
+                        &&  ( (x+sdx >0) && (x+sdx < the_map.width) &&  /* only inside map */
+                              (y+sdy >0) && (y+sdy < the_map.height) ) ) {
+                    mapdata_cell(x+sdx, y+sdy)->need_resmooth=1;
                 }
     }
 }
@@ -215,21 +212,21 @@ static void expand_clear_face(int x, int y, int w, int h, int layer)
     int dx, dy;
     struct MapCell *cell;
 
-    assert(0 <= x && x < FOG_MAP_SIZE);
-    assert(0 <= y && y < FOG_MAP_SIZE);
+    assert(0 <= x && x < the_map.width);
+    assert(0 <= y && y < the_map.height);
     assert(1 <= w && w <= MAX_FACE_SIZE);
     assert(1 <= h && h <= MAX_FACE_SIZE);
 
-    assert(0 <= x-w+1 && x-w+1 < FOG_MAP_SIZE);
-    assert(0 <= y-h+1 && y-h+1 < FOG_MAP_SIZE);
+    assert(0 <= x-w+1 && x-w+1 < the_map.width);
+    assert(0 <= y-h+1 && y-h+1 < the_map.height);
 
     cell = mapdata_cell(x, y);
 
     for (dx = 0; dx < w; dx++) {
         for (dy = !dx; dy < h; dy++) {
-            struct MapCellLayer *tail = &the_map.cells[x-dx][y-dy].tails[layer];
-            assert(0 <= x-dx && x-dx < FOG_MAP_SIZE);
-            assert(0 <= y-dy && y-dy < FOG_MAP_SIZE);
+            struct MapCellTailLayer *tail = &mapdata_cell(x-dx, y-dy)->tails[layer];
+            assert(0 <= x-dx && x-dx < the_map.width);
+            assert(0 <= y-dy && y-dy < the_map.height);
             assert(0 <= layer && layer < MAXLAYERS);
 
             /* Do not clear faces that already have been overwritten by another
@@ -241,7 +238,7 @@ static void expand_clear_face(int x, int y, int w, int h, int layer)
                 tail->face = 0;
                 tail->size_x = 0;
                 tail->size_y = 0;
-                the_map.cells[x-dx][y-dy].need_update = 1;
+                mapdata_cell(x-dx, y-dy)->need_update = 1;
             }
             mark_resmooth(x-dx,y-dy,layer);
         }
@@ -269,8 +266,8 @@ static void expand_clear_face_from_layer(int x, int y, int layer)
 {
     const struct MapCellLayer *cell;
 
-    assert(0 <= x && x < FOG_MAP_SIZE);
-    assert(0 <= y && y < FOG_MAP_SIZE);
+    assert(0 <= x && x < the_map.width);
+    assert(0 <= y && y < the_map.height);
     assert(0 <= layer && layer < MAXLAYERS);
 
     cell = &mapdata_cell(x, y)->heads[layer];
@@ -298,8 +295,8 @@ static void expand_set_face(int x, int y, int layer, gint16 face, int clear)
     int dx, dy;
     guint8 w, h;
 
-    assert(0 <= x && x < FOG_MAP_SIZE);
-    assert(0 <= y && y < FOG_MAP_SIZE);
+    assert(0 <= x && x < the_map.width);
+    assert(0 <= y && y < the_map.height);
     assert(0 <= layer && layer < MAXLAYERS);
 
     cell = mapdata_cell(x, y);
@@ -319,15 +316,15 @@ static void expand_set_face(int x, int y, int layer, gint16 face, int clear)
 
     for (dx = 0; dx < w; dx++) {
         for (dy = !dx; dy < h; dy++) {
-            struct MapCellLayer *tail = &the_map.cells[x-dx][y-dy].tails[layer];
-            assert(0 <= x-dx && x-dx < FOG_MAP_SIZE);
-            assert(0 <= y-dy && y-dy < FOG_MAP_SIZE);
+            struct MapCellTailLayer *tail = &mapdata_cell(x-dx, y-dy)->tails[layer];
+            assert(0 <= x-dx && x-dx < the_map.width);
+            assert(0 <= y-dy && y-dy < the_map.height);
             assert(0 <= layer && layer < MAXLAYERS);
 
             tail->face = face;
             tail->size_x = dx;
             tail->size_y = dy;
-            the_map.cells[x-dx][y-dy].need_update = 1;
+            mapdata_cell(x-dx, y-dy)->need_update = 1;
             mark_resmooth(x-dx,y-dy,layer);
         }
     }
@@ -357,7 +354,7 @@ static void expand_clear_bigface(int x, int y, int w, int h, int layer, int set_
 
     for (dx = 0; dx < w && dx <= x; dx++) {
         for (dy = !dx; dy < h && dy <= y; dy++) {
-            struct MapCellLayer *tail = &bigfaces[x-dx][y-dy][layer].tail;
+            struct MapCellTailLayer *tail = &bigfaces[x-dx][y-dy][layer].tail;
             assert(0 <= x-dx && x-dx < MAX_VIEW);
             assert(0 <= y-dy && y-dy < MAX_VIEW);
             assert(0 <= layer && layer < MAXLAYERS);
@@ -374,10 +371,10 @@ static void expand_clear_bigface(int x, int y, int w, int h, int layer, int set_
 
                 if (0 <= x-dx && x-dx < width
                         && 0 <= y-dy && y-dy < height) {
-                    assert(0 <= pl_pos.x+x-dx && pl_pos.x+x-dx < FOG_MAP_SIZE);
-                    assert(0 <= pl_pos.y+y-dy && pl_pos.y+y-dy < FOG_MAP_SIZE);
+                    assert(0 <= pl_pos.x+x-dx && pl_pos.x+x-dx < the_map.width);
+                    assert(0 <= pl_pos.y+y-dy && pl_pos.y+y-dy < the_map.height);
                     if (set_need_update) {
-                        the_map.cells[pl_pos.x+x-dx][pl_pos.y+y-dy].need_update = 1;
+                        mapdata_cell(pl_pos.x+x-dx, pl_pos.y+y-dy)->need_update = 1;
                     }
                 }
             }
@@ -482,7 +479,7 @@ static void expand_set_bigface(int x, int y, int layer, gint16 face, int clear)
 
     for (dx = 0; dx < w && dx <= x; dx++) {
         for (dy = !dx; dy < h && dy <= y; dy++) {
-            struct MapCellLayer *tail = &bigfaces[x-dx][y-dy][layer].tail;
+            struct MapCellTailLayer *tail = &bigfaces[x-dx][y-dy][layer].tail;
             assert(0 <= x-dx && x-dx < MAX_VIEW);
             assert(0 <= y-dy && y-dy < MAX_VIEW);
             assert(0 <= layer && layer < MAXLAYERS);
@@ -493,9 +490,9 @@ static void expand_set_bigface(int x, int y, int layer, gint16 face, int clear)
 
             if (0 <= x-dx && x-dx < width
                     && 0 <= y-dy && y-dy < height) {
-                assert(0 <= pl_pos.x+x-dx && pl_pos.x+x-dx < FOG_MAP_SIZE);
-                assert(0 <= pl_pos.y+y-dy && pl_pos.y+y-dy < FOG_MAP_SIZE);
-                the_map.cells[pl_pos.x+x-dx][pl_pos.y+y-dy].need_update = 1;
+                assert(0 <= pl_pos.x+x-dx && pl_pos.x+x-dx < the_map.width);
+                assert(0 <= pl_pos.y+y-dy && pl_pos.y+y-dy < the_map.height);
+                mapdata_cell(pl_pos.x+x-dx, pl_pos.y+y-dy)->need_update = 1;
             }
         }
     }
@@ -512,19 +509,19 @@ static void expand_need_update(int x, int y, int w, int h)
 {
     int dx, dy;
 
-    assert(0 <= x && x < FOG_MAP_SIZE);
-    assert(0 <= y && y < FOG_MAP_SIZE);
+    assert(0 <= x && x < the_map.width);
+    assert(0 <= y && y < the_map.height);
     assert(1 <= w && w <= MAX_FACE_SIZE);
     assert(1 <= h && h <= MAX_FACE_SIZE);
 
-    assert(0 <= x-w+1 && x-w+1 < FOG_MAP_SIZE);
-    assert(0 <= y-h+1 && y-h+1 < FOG_MAP_SIZE);
+    assert(0 <= x-w+1 && x-w+1 < the_map.width);
+    assert(0 <= y-h+1 && y-h+1 < the_map.height);
 
     for (dx = 0; dx < w; dx++) {
         for (dy = 0; dy < h; dy++) {
-            struct MapCell *cell = &the_map.cells[x-dx][y-dy];
-            assert(0 <= x-dx && x-dx < FOG_MAP_SIZE);
-            assert(0 <= y-dy && y-dy < FOG_MAP_SIZE);
+            struct MapCell *cell = mapdata_cell(x-dx, y-dy);
+            assert(0 <= x-dx && x-dx < the_map.width);
+            assert(0 <= y-dy && y-dy < the_map.height);
             cell->need_update = 1;
         }
     }
@@ -540,8 +537,8 @@ static void expand_need_update_from_layer(int x, int y, int layer)
 {
     struct MapCellLayer *head;
 
-    assert(0 <= x && x < FOG_MAP_SIZE);
-    assert(0 <= y && y < FOG_MAP_SIZE);
+    assert(0 <= x && x < the_map.width);
+    assert(0 <= y && y < the_map.height);
     assert(0 <= layer && layer < MAXLAYERS);
 
     head = &mapdata_cell(x, y)->heads[layer];
@@ -553,42 +550,43 @@ static void expand_need_update_from_layer(int x, int y, int layer)
     }
 }
 
-void mapdata_init(void)
+/**
+ * Allocate and set up pointers for a map, with cells represented as a C-style
+ * multi-dimensional array.
+ */
+static void mapdata_alloc(struct Map* const map, const int width, const int height) {
+    map->_cells = (struct MapCell **)g_new(struct MapCell, width * (height + 1));
+    g_assert(map->_cells != NULL); // g_new() always succeeds
+    map->width = width;
+    map->height = height;
+
+    /* Skip past the first row of pointers to rows and assign the
+     * start of the actual map data
+     */
+    map->_cells[0] = (struct MapCell *)((char *)map->_cells+(sizeof(struct MapCell *)*map->width));
+
+    /* Finish assigning the beginning of each row relative to the
+     * first row assigned above
+     */
+    for (int i = 0; i < map->width; i++) {
+        map->_cells[i] = map->_cells[0]+i*map->height;
+    }
+}
+
+static void mapdata_init(void)
 {
     int x, y;
     int i;
 
-    if (the_map.cells == NULL) {
-        the_map.cells = g_malloc(
-                            sizeof(*the_map.cells)*FOG_MAP_SIZE+
-                            sizeof(**the_map.cells)*FOG_MAP_SIZE*FOG_MAP_SIZE);
-        if (the_map.cells == NULL) {
-            LOG(LOG_ERROR, "mapdata_init", "%s\n", "out of memory");
-            exit(1);
-        }
-
-        /* Skip past the first row of pointers to rows and assign the
-         * start of the actual map data
-         */
-        the_map.cells[0] = (struct MapCell *)((char *)the_map.cells+(sizeof(struct MapCell *)*FOG_MAP_SIZE));
-
-        /* Finish assigning the beginning of each row relative to the
-         * first row assigned above
-         */
-        for (i = 0; i < FOG_MAP_SIZE; i++) {
-            the_map.cells[i] = the_map.cells[0]+i*FOG_MAP_SIZE;
-        }
-        the_map.x = FOG_MAP_SIZE;
-        the_map.y = FOG_MAP_SIZE;
-    }
+    mapdata_alloc(&the_map, FOG_MAP_SIZE, FOG_MAP_SIZE);
 
     width = 0;
     height = 0;
-    pl_pos.x = FOG_MAP_SIZE/2-width/2;
-    pl_pos.y = FOG_MAP_SIZE/2-height/2;
+    pl_pos.x = the_map.width/2-width/2;
+    pl_pos.y = the_map.height/2-height/2;
 
-    for (x = 0; x < FOG_MAP_SIZE; x++) {
-        clear_cells(x, 0, FOG_MAP_SIZE);
+    for (x = 0; x < the_map.width; x++) {
+        clear_cells(x, 0, the_map.height);
     }
 
     for (y = 0; y < MAX_VIEW; y++) {
@@ -611,14 +609,22 @@ void mapdata_init(void)
     bigfaces_head = NULL;
 }
 
+void mapdata_free(void) {
+    if (the_map._cells != NULL) {
+        g_free(the_map._cells);
+        the_map._cells = NULL;
+    }
+}
+
 void mapdata_set_size(int viewx, int viewy)
 {
+    mapdata_free();
     mapdata_init();
 
     width = viewx;
     height = viewy;
-    pl_pos.x = FOG_MAP_SIZE/2-width/2;
-    pl_pos.y = FOG_MAP_SIZE/2-height/2;
+    pl_pos.x = the_map.width/2-width/2;
+    pl_pos.y = the_map.height/2-height/2;
 }
 
 int mapdata_is_inside(int x, int y)
@@ -640,8 +646,8 @@ void mapdata_clear_space(int x, int y)
 
     px = pl_pos.x+x;
     py = pl_pos.y+y;
-    assert(0 <= px && px < FOG_MAP_SIZE);
-    assert(0 <= py && py < FOG_MAP_SIZE);
+    assert(0 <= px && px < the_map.width);
+    assert(0 <= py && py < the_map.height);
 
     if (x < width && y < height) {
         /* tile is visible */
@@ -685,8 +691,8 @@ void mapdata_set_check_space(int x, int y)
     px = pl_pos.x+x;
     py = pl_pos.y+y;
 
-    assert(0 <= px && px < FOG_MAP_SIZE);
-    assert(0 <= py && py < FOG_MAP_SIZE);
+    assert(0 <= px && px < the_map.width);
+    assert(0 <= py && py < the_map.height);
 
 
     is_blank=1;
@@ -736,8 +742,8 @@ void mapdata_set_darkness(int x, int y, int darkness)
 
     px = pl_pos.x+x;
     py = pl_pos.y+y;
-    assert(0 <= px && px < FOG_MAP_SIZE);
-    assert(0 <= py && py < FOG_MAP_SIZE);
+    assert(0 <= px && px < the_map.width);
+    assert(0 <= py && py < the_map.height);
 
     /* Ignore darkness information for tile outside the viewable area: if
      * such a tile becomes visible again, it is either "fog of war" (and
@@ -750,7 +756,7 @@ void mapdata_set_darkness(int x, int y, int darkness)
 }
 
 /* Sets smooth information for layer */
-void mapdata_set_smooth(int x, int y, int smooth, int layer)
+void mapdata_set_smooth(int x, int y, guint8 smooth, int layer)
 {
     static int dx[8]= {0,1,1,1,0,-1,-1,-1};
     static int dy[8]= {-1,-1,0,1,1,1,0,-1};
@@ -761,17 +767,17 @@ void mapdata_set_smooth(int x, int y, int smooth, int layer)
 
     px = pl_pos.x+x;
     py = pl_pos.y+y;
-    assert(0 <= px && px < FOG_MAP_SIZE);
-    assert(0 <= py && py < FOG_MAP_SIZE);
+    assert(0 <= px && px < the_map.width);
+    assert(0 <= py && py < the_map.height);
 
     if (mapdata_cell(px, py)->smooth[layer] != smooth) {
         for (i=0; i<8; i++) {
             rx=px+dx[i];
             ry=py+dy[i];
-            if ( (rx<0) || (ry<0) || (the_map.x<=rx) || (the_map.y<=ry)) {
+            if ( (rx<0) || (ry<0) || (the_map.width<=rx) || (the_map.height<=ry)) {
                 continue;
             }
-            the_map.cells[rx][ry].need_resmooth=1;
+            mapdata_cell(rx, ry)->need_resmooth=1;
         }
         mapdata_cell(px, py)->need_resmooth=1;
         mapdata_cell(px, py)->smooth[layer] = smooth;
@@ -794,8 +800,8 @@ void mapdata_clear_old(int x, int y)
 
     px = pl_pos.x+x;
     py = pl_pos.y+y;
-    assert(0 <= px && px < FOG_MAP_SIZE);
-    assert(0 <= py && py < FOG_MAP_SIZE);
+    assert(0 <= px && px < the_map.width);
+    assert(0 <= py && py < the_map.height);
 
     if (x < width && y < height)
         if (mapdata_cell(px, py)->cleared) {
@@ -821,8 +827,8 @@ void mapdata_set_face_layer(int x, int y, gint16 face, int layer)
 
     px = pl_pos.x+x;
     py = pl_pos.y+y;
-    assert(0 <= px && px < FOG_MAP_SIZE);
-    assert(0 <= py && py < FOG_MAP_SIZE);
+    assert(0 <= px && px < the_map.width);
+    assert(0 <= py && py < the_map.height);
 
     if (x < width && y < height) {
         mapdata_cell(px, py)->need_update = 1;
@@ -853,8 +859,8 @@ void mapdata_set_anim_layer(int x, int y, guint16 anim, guint8 anim_speed, int l
 
     px = pl_pos.x+x;
     py = pl_pos.y+y;
-    assert(0 <= px && px < FOG_MAP_SIZE);
-    assert(0 <= py && py < FOG_MAP_SIZE);
+    assert(0 <= px && px < the_map.width);
+    assert(0 <= py && py < the_map.height);
 
     animation = anim & ANIM_MASK;
     face = 0;
@@ -922,7 +928,7 @@ void mapdata_scroll(int dx, int dy)
                 for (y = !x; y < cell->head.size_y; y++) {
                     if (0 <= cell->x-x && cell->x-x < width
                             && 0 <= cell->y-y && cell->y-y < height) {
-                        the_map.cells[pl_pos.x+cell->x-x][pl_pos.y+cell->y-y].need_update = 1;
+                        mapdata_cell(pl_pos.x+cell->x-x, pl_pos.y+cell->y-y)->need_update = 1;
                     }
                 }
             }
@@ -931,7 +937,7 @@ void mapdata_scroll(int dx, int dy)
         /* Emulate map scrolling by redrawing all tiles. */
         for (x = 0; x < width; x++) {
             for (y = 0; y < height; y++) {
-                the_map.cells[pl_pos.x+x][pl_pos.y+y].need_update = 1;
+                mapdata_cell(pl_pos.x+x, pl_pos.y+y)->need_update = 1;
             }
         }
     }
@@ -943,15 +949,15 @@ void mapdata_scroll(int dx, int dy)
     if (dx > 0) {
         for (y = 0; y < height; y++) {
             for (x = width-dx; x < width; x++) {
-                the_map.cells[pl_pos.x+x][pl_pos.y+y].cleared = 1;
-                the_map.cells[pl_pos.x+x][pl_pos.y+y].need_update = 1;
+                mapdata_cell(pl_pos.x+x, pl_pos.y+y)->cleared = 1;
+                mapdata_cell(pl_pos.x+x, pl_pos.y+y)->need_update = 1;
             }
         }
     } else {
         for (y = 0; y < height; y++) {
             for (x = 0; x < -dx; x++) {
-                the_map.cells[pl_pos.x+x][pl_pos.y+y].cleared = 1;
-                the_map.cells[pl_pos.x+x][pl_pos.y+y].need_update = 1;
+                mapdata_cell(pl_pos.x+x, pl_pos.y+y)->cleared = 1;
+                mapdata_cell(pl_pos.x+x, pl_pos.y+y)->need_update = 1;
             }
         }
     }
@@ -959,15 +965,15 @@ void mapdata_scroll(int dx, int dy)
     if (dy > 0) {
         for (x = 0; x < width; x++) {
             for (y = height-dy; y < height; y++) {
-                the_map.cells[pl_pos.x+x][pl_pos.y+y].cleared = 1;
-                the_map.cells[pl_pos.x+x][pl_pos.y+y].need_update = 1;
+                mapdata_cell(pl_pos.x+x, pl_pos.y+y)->cleared = 1;
+                mapdata_cell(pl_pos.x+x, pl_pos.y+y)->need_update = 1;
             }
         }
     } else {
         for (x = 0; x < width; x++) {
             for (y = 0; y < -dy; y++) {
-                the_map.cells[pl_pos.x+x][pl_pos.y+y].cleared = 1;
-                the_map.cells[pl_pos.x+x][pl_pos.y+y].need_update = 1;
+                mapdata_cell(pl_pos.x+x, pl_pos.y+y)->cleared = 1;
+                mapdata_cell(pl_pos.x+x, pl_pos.y+y)->need_update = 1;
             }
         }
     }
@@ -983,9 +989,9 @@ void mapdata_newmap(void)
     int x, y;
 
     /* Clear the_map.cells[]. */
-    for (x = 0; x < FOG_MAP_SIZE; x++) {
-        clear_cells(x, 0, FOG_MAP_SIZE);
-        for (y = 0; y < FOG_MAP_SIZE; y++) {
+    for (x = 0; x < the_map.width; x++) {
+        clear_cells(x, 0, the_map.height);
+        for (y = 0; y < the_map.height; y++) {
             mapdata_cell(x, y)->need_update = 1;
         }
     }
@@ -1021,18 +1027,18 @@ gint16 mapdata_face(int x, int y, int layer) {
         return 0;
     }
 
-    return the_map.cells[pl_pos.x+x][pl_pos.y+y].heads[layer].face;
+    return mapdata_cell(pl_pos.x+x, pl_pos.y+y)->heads[layer].face;
 }
 
 gint16 mapdata_face_info(int mx, int my, int layer, int *dx, int *dy) {
-    struct MapCellLayer *head = &the_map.cells[mx][my].heads[layer];
-    struct MapCellLayer *tail = &the_map.cells[mx][my].tails[layer];
+    struct MapCellLayer *head = &mapdata_cell(mx, my)->heads[layer];
+    struct MapCellTailLayer *tail = &mapdata_cell(mx, my)->tails[layer];
     if (head->face != 0) {
         const int width = head->size_x, height = head->size_y;
         *dx = 1 - width, *dy = 1 - height;
         return head->face;
     } else if (tail->face != 0) {
-        struct MapCellLayer *head_ptr = &the_map.cells[mx + tail->size_x][my + tail->size_y].heads[layer];
+        struct MapCellLayer *head_ptr = &mapdata_cell(mx + tail->size_x, my + tail->size_y)->heads[layer];
         const int width = head_ptr->size_x, height = head_ptr->size_y;
         *dx = tail->size_x - width + 1, *dy = tail->size_y - height + 1;
         return tail->face;
@@ -1057,13 +1063,13 @@ gint16 mapdata_bigface(int x, int y, int layer, int *ww, int *hh) {
         return 0;
     }
 
-    result = the_map.cells[pl_pos.x+x][pl_pos.y+y].tails[layer].face;
+    result = mapdata_cell(pl_pos.x+x, pl_pos.y+y)->tails[layer].face;
     if (result != 0) {
         int clear_bigface;
-        int dx = the_map.cells[pl_pos.x+x][pl_pos.y+y].tails[layer].size_x;
-        int dy = the_map.cells[pl_pos.x+x][pl_pos.y+y].tails[layer].size_y;
-        int w = the_map.cells[pl_pos.x+x+dx][pl_pos.y+y+dy].heads[layer].size_x;
-        int h = the_map.cells[pl_pos.x+x+dx][pl_pos.y+y+dy].heads[layer].size_y;
+        int dx = mapdata_cell(pl_pos.x+x, pl_pos.y+y)->tails[layer].size_x;
+        int dy = mapdata_cell(pl_pos.x+x, pl_pos.y+y)->tails[layer].size_y;
+        int w = mapdata_cell(pl_pos.x+x+dx, pl_pos.y+y+dy)->heads[layer].size_x;
+        int h = mapdata_cell(pl_pos.x+x+dx, pl_pos.y+y+dy)->heads[layer].size_y;
         assert(1 <= w && w <= MAX_FACE_SIZE);
         assert(1 <= h && h <= MAX_FACE_SIZE);
         assert(0 <= dx && dx < w);
@@ -1075,7 +1081,7 @@ gint16 mapdata_bigface(int x, int y, int layer, int *ww, int *hh) {
          * tile if it was already valid, just clear the big face and do not
          * return it.
          */
-        if (the_map.cells[pl_pos.x+x][pl_pos.y+y].cleared) {
+        if (mapdata_cell(pl_pos.x+x, pl_pos.y+y)->cleared) {
             /* Current face is a "fog of war" tile ==> do not clear
              * old information.
              */
@@ -1085,7 +1091,7 @@ gint16 mapdata_bigface(int x, int y, int layer, int *ww, int *hh) {
                 /* Clear face if current tile is valid but the
                  * head is marked as cleared.
                  */
-                clear_bigface = the_map.cells[pl_pos.x+x+dx][pl_pos.y+y+dy].cleared;
+                clear_bigface = mapdata_cell(pl_pos.x+x+dx, pl_pos.y+y+dy)->cleared;
             } else {
                 /* Clear face if current tile is valid but the
                  * head is not set.
@@ -1100,9 +1106,9 @@ gint16 mapdata_bigface(int x, int y, int layer, int *ww, int *hh) {
             return(result);
         }
 
-        assert(the_map.cells[pl_pos.x+x][pl_pos.y+y].tails[layer].face == result);
+        assert(mapdata_cell(pl_pos.x+x, pl_pos.y+y)->tails[layer].face == result);
         expand_clear_face_from_layer(pl_pos.x+x+dx, pl_pos.y+y+dy, layer);
-        assert(the_map.cells[pl_pos.x+x][pl_pos.y+y].tails[layer].face == 0);
+        assert(mapdata_cell(pl_pos.x+x, pl_pos.y+y)->tails[layer].face == 0);
     }
 
     result = bigfaces[x][y][layer].tail.face;
@@ -1188,10 +1194,10 @@ static void recenter_virtual_map_view(int diff_x, int diff_y)
          * i.e. left border is FOG_BORDER_MIN+MAX_FACE_SIZE after
          * shifting.
          */
-    } else if (new_x+MAX_VIEW > FOG_MAP_SIZE) {
-        shift_x = FOG_MAP_SIZE-FOG_BORDER_MIN-MAX_VIEW-new_x;
+    } else if (new_x+MAX_VIEW > the_map.width) {
+        shift_x = the_map.width-FOG_BORDER_MIN-MAX_VIEW-new_x;
         /* This yields: new_x+shift_x ==
-         * FOG_MAP_SIZE-FOG_BODER_MIN-MAX_VIEW, i.e. right border is
+         * the_map.width-FOG_BODER_MIN-MAX_VIEW, i.e. right border is
          * FOGBORDER_MIN after shifting.
          */
     } else {
@@ -1201,8 +1207,8 @@ static void recenter_virtual_map_view(int diff_x, int diff_y)
     /* Same as above but for y. */
     if (new_y < MAX_FACE_SIZE) {
         shift_y = FOG_BORDER_MIN+MAX_FACE_SIZE-new_y;
-    } else if (new_y+MAX_VIEW > FOG_MAP_SIZE) {
-        shift_y = FOG_MAP_SIZE-FOG_BORDER_MIN-MAX_VIEW-new_y;
+    } else if (new_y+MAX_VIEW > the_map.height) {
+        shift_y = the_map.height-FOG_BORDER_MIN-MAX_VIEW-new_y;
     } else {
         shift_y = 0;
     }
@@ -1220,29 +1226,29 @@ static void recenter_virtual_map_view(int diff_x, int diff_y)
     if (shift_x == 0) {
         if (new_x < FOG_BORDER_MIN+MAX_FACE_SIZE) {
             shift_x = FOG_BORDER_MIN+MAX_FACE_SIZE-new_x;
-        } else if (new_x+MAX_VIEW+FOG_BORDER_MIN > FOG_MAP_SIZE) {
-            shift_x = FOG_MAP_SIZE-FOG_BORDER_MIN-MAX_VIEW-new_x;
+        } else if (new_x+MAX_VIEW+FOG_BORDER_MIN > the_map.width) {
+            shift_x = the_map.width-FOG_BORDER_MIN-MAX_VIEW-new_x;
         }
     }
     if (shift_y == 0) {
         if (new_y < FOG_BORDER_MIN+MAX_FACE_SIZE) {
             shift_y = FOG_BORDER_MIN+MAX_FACE_SIZE-new_y;
-        } else if (new_y+MAX_VIEW+FOG_BORDER_MIN > FOG_MAP_SIZE) {
-            shift_y = FOG_MAP_SIZE-FOG_BORDER_MIN-MAX_VIEW-new_y;
+        } else if (new_y+MAX_VIEW+FOG_BORDER_MIN > the_map.height) {
+            shift_y = the_map.height-FOG_BORDER_MIN-MAX_VIEW-new_y;
         }
     }
 
     /* Shift for more than virtual map size? ==> clear whole virtual map
      * and recenter.
      */
-    if (shift_x <= -FOG_MAP_SIZE || shift_x >= FOG_MAP_SIZE
-            || shift_y <= -FOG_MAP_SIZE || shift_y >= FOG_MAP_SIZE) {
-        for (dx = 0; dx < FOG_MAP_SIZE; dx++) {
-            clear_cells(dx, 0, FOG_MAP_SIZE);
+    if (shift_x <= -the_map.width || shift_x >= the_map.width
+            || shift_y <= -the_map.height || shift_y >= the_map.height) {
+        for (dx = 0; dx < the_map.width; dx++) {
+            clear_cells(dx, 0, the_map.height);
         }
 
-        pl_pos.x = FOG_MAP_SIZE/2-width/2;
-        pl_pos.y = FOG_MAP_SIZE/2-height/2;
+        pl_pos.x = the_map.width/2-width/2;
+        pl_pos.y = the_map.height/2-height/2;
         return;
     }
 
@@ -1254,21 +1260,21 @@ static void recenter_virtual_map_view(int diff_x, int diff_y)
     if (shift_x < 0) {
         src_x = -shift_x;
         dst_x = 0;
-        len_x = FOG_MAP_SIZE+shift_x;
+        len_x = the_map.width+shift_x;
     } else {
         src_x = 0;
         dst_x = shift_x;
-        len_x = FOG_MAP_SIZE-shift_x;
+        len_x = the_map.width-shift_x;
     }
 
     if (shift_y < 0) {
         src_y = -shift_y;
         dst_y = 0;
-        len_y = FOG_MAP_SIZE+shift_y;
+        len_y = the_map.height+shift_y;
     } else {
         src_y = 0;
         dst_y = shift_y;
-        len_y = FOG_MAP_SIZE-shift_y;
+        len_y = the_map.height-shift_y;
     }
 
     if (shift_x < 0) {
@@ -1276,14 +1282,14 @@ static void recenter_virtual_map_view(int diff_x, int diff_y)
             /* srcx!=dstx ==> can use memcpy since source and
              * destination to not overlap.
              */
-            memcpy(&the_map.cells[dx][dst_y], &the_map.cells[sx][src_y], len_y*sizeof(the_map.cells[dx][dst_y]));
+            memcpy(mapdata_cell(dx, dst_y), mapdata_cell(sx, src_y), len_y*sizeof(struct MapCell));
         }
     } else if (shift_x > 0) {
         for (sx = src_x+len_x-1, dx = dst_x+len_x-1, i = 0; i < len_x; sx--, dx--, i++) {
             /* srcx!=dstx ==> can use memcpy since source and
              * destination to not overlap.
              */
-            memcpy(&the_map.cells[dx][dst_y], &the_map.cells[sx][src_y], len_y*sizeof(the_map.cells[dx][dst_y]));
+            memcpy(mapdata_cell(dx, dst_y), mapdata_cell(sx, src_y), len_y*sizeof(struct MapCell));
         }
     } else {
         assert(src_x == dst_x);
@@ -1291,16 +1297,16 @@ static void recenter_virtual_map_view(int diff_x, int diff_y)
             /* srcx==dstx ==> use memmove since source and
              * destination probably do overlap.
              */
-            memmove(&the_map.cells[dx][dst_y], &the_map.cells[dx][src_y], len_y*sizeof(the_map.cells[dx][dst_y]));
+            memmove(mapdata_cell(dx, dst_y), mapdata_cell(dx, src_y), len_y*sizeof(struct MapCell));
         }
     }
 
     /* Clear newly opened area */
     for (dx = 0; dx < dst_x; dx++) {
-        clear_cells(dx, 0, FOG_MAP_SIZE);
+        clear_cells(dx, 0, the_map.height);
     }
-    for (dx = dst_x+len_x; dx < FOG_MAP_SIZE; dx++) {
-        clear_cells(dx, 0, FOG_MAP_SIZE);
+    for (dx = dst_x+len_x; dx < the_map.width; dx++) {
+        clear_cells(dx, 0, the_map.height);
     }
     if (shift_y > 0) {
         for (dx = 0; dx < len_x; dx++) {
@@ -1308,7 +1314,7 @@ static void recenter_virtual_map_view(int diff_x, int diff_y)
         }
     } else if (shift_y < 0) {
         for (dx = 0; dx < len_x; dx++) {
-            clear_cells(dx+dst_x, FOG_MAP_SIZE+shift_y, -shift_y);
+            clear_cells(dx+dst_x, the_map.height+shift_y, -shift_y);
         }
     }
 }
@@ -1368,13 +1374,13 @@ void mapdata_animation(void)
             /* Short cut some processing here.  It makes sense to me
              * not to animate stuff out of view
              */
-            if (the_map.cells[pl_pos.x + x][pl_pos.y + y].cleared) {
+            if (mapdata_cell(pl_pos.x + x, pl_pos.y + y)->cleared) {
                 continue;
             }
 
             for (layer=0; layer<MAXLAYERS; layer++) {
                 /* Using the cell structure just makes life easier here */
-                cell = &the_map.cells[pl_pos.x+x][pl_pos.y+y].heads[layer];
+                cell = &mapdata_cell(pl_pos.x+x, pl_pos.y+y)->heads[layer];
 
                 if (cell->animation) {
                     cell->animation_left++;
