@@ -33,6 +33,12 @@ struct MapCellLayer {
     guint8   animation_phase;
 };
 
+struct MapCellTailLayer {
+    gint16 face;
+    gint8 size_x;
+    gint8 size_y;
+};
+
 /** The heads[] in the mapcell is used for single part objects
  * or the head piece for multipart.  The easiest way to think about
  * it is that the heads[] contains the map information as specifically
@@ -49,8 +55,8 @@ struct MapCellLayer {
 struct MapCell
 {
     struct MapCellLayer heads[MAXLAYERS];
-    struct MapCellLayer tails[MAXLAYERS];
-    guint16 smooth[MAXLAYERS];
+    struct MapCellTailLayer tails[MAXLAYERS];
+    guint8 smooth[MAXLAYERS];
     guint8 darkness;         /* darkness: 0=fully illuminated, 255=pitch black */
     guint8 need_update:1;    /* set if tile should be redrawn */
     guint8 have_darkness:1;  /* set if darkness information was set */
@@ -60,13 +66,9 @@ struct MapCell
 
 struct Map
 {
-    /* Store size of map so we know if map_size has changed
-     * since the last time we allocated this;
-     */
-    int x;
-    int y;
-
-    struct MapCell **cells;
+    struct MapCell **_cells; //< data, access via mapdata_cells()
+    int width;  //< width of cells array
+    int height; //< height of cells array
 };
 
 struct MapCell *mapdata_cell(int x, int y);
@@ -76,16 +78,17 @@ bool mapdata_can_smooth(int x, int y, int layer);
 
 /**
  * Initializes the module. Allocates memory for the_map. This functions must be
- * called before any other function is used.
- */
-void mapdata_init(void);
-
-/**
- * Sets the current display size. Must be called whenever a new display size
+ * called before any other function is used, and whenever a new display size
  * was negotiated with the server.
  */
 void mapdata_set_size(int viewx, int viewy);
 
+/**
+ * Deallocate map data. Do not call functions other than mapdata_set_size()
+ * after calling mapdata_free().
+ */
+void mapdata_free(void);
+
 /**
  * Scrolls the map view. Must be called whenever a map_scroll command was
  * received from the server.
@@ -141,7 +144,7 @@ gint16 mapdata_bigface(int x, int y, int layer, int *ww, int *hh);
 void mapdata_clear_space(int x, int y);
 void mapdata_set_check_space(int x, int y);
 void mapdata_set_darkness(int x, int y, int darkness);
-void mapdata_set_smooth(int x, int y, int smooth, int layer);
+void mapdata_set_smooth(int x, int y, guint8 smooth, int layer);
 void mapdata_clear_old(int x, int y);
 void mapdata_set_face_layer(int x, int y, gint16 face, int layer);
 void mapdata_set_anim_layer(int x, int y, guint16 anim, guint8 anim_speed, int layer);
