@@ -12,159 +12,65 @@
  */
 
 /**
- * @file common/p_cmd.c
+ * @file
  * Contains a lot about the commands typed into the client.
  */
 
-#ifndef CPROTO
-/* use declarations from p_cmd.h instead of doing make proto on this file */
+#include <ctype.h>
 
-#include <client.h>
-#include <external.h>
-#include <script.h>
-#include <p_cmd.h>
+#include "client.h"
+#include "external.h"
+#include "p_cmd.h"
+#include "script.h"
 
 /**
  * @defgroup PCmdHelpCommands Common client player commands.
  * @{
  */
 
-/* TODO This should really be under /help commands or something... */
-
-/* This dynamically generates a list from the ConsoleCommand list. */
-#undef CLIENTHELP_LONG_LIST
-
-/*
-long-list:
-category
-name - description
-name - description
-...
-
-not long list:
-category
-name name name ...
-*/
-
-#undef HELP_USE_COLOR
-#ifdef HELP_USE_COLOR
-#error Oops, need to put them back.
-#else
 #define H1(a) draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE, a)
 #define H2(a) draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE, a)
 #define LINE(a) draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE, a)
-#endif
-
-#define assumed_wrap get_info_width()
 
 /* TODO Help topics other than commands? Refer to other documents? */
 
-/**
- *
- */
-static void do_clienthelp_list(void)
-{
-    ConsoleCommand ** commands_array;
-    ConsoleCommand * commands_copy;
-    int i;
-    CommCat current_cat = COMM_CAT_MISC;
-#ifndef CLIENTHELP_LONG_LIST
-    char line_buf[MAX_BUF];
-    size_t line_len = 0;
+static int get_num_commands(void);
 
-    line_buf[0] = '\0';
-#endif
+static void do_clienthelp_list() {
+    ConsoleCommand **sorted_cmds = get_cat_sorted_commands();
+    CommCat category = COMM_CAT_MISC;
+    GString *line = g_string_new(NULL);
 
-    commands_array = get_cat_sorted_commands();
-
-    /* Now we have a nice sorted list. */
-
-    H1(" === Client Side Commands === ");
-
-    for (i = 0; i < get_num_commands(); i++) {
-        commands_copy = commands_array[i];
-
-        /* Should be LOG_SPAM but I'm too lazy to tweak it. */
-        /* LOG(LOG_INFO, "p_cmd::do_clienthelp_list", "%s Command %s", get_category_name(commands_copy->cat), commands_copy->name); */
-
-        if (commands_copy->cat != current_cat) {
+    H1("Client commands:");
+    for (int i = 0; i < get_num_commands(); i++) {
+        ConsoleCommand *cmd = sorted_cmds[i];
+        if (cmd->cat != category) {
+            // If moving on to next category, dump line_buf and print header.
             char buf[MAX_BUF];
-
-#ifndef CLIENTHELP_LONG_LIST
-            if (line_len > 0) {
-                LINE(line_buf);
-                line_buf[0] = '\0';
-                line_len = 0;
-            }
-#endif
-
-#ifdef HELP_USE_COLOR
-            snprintf(buf, MAX_BUF - 1, "%s Commands:", get_category_name(commands_copy->cat));
-#else
-            snprintf(buf, MAX_BUF - 1, " --- %s Commands --- ", get_category_name(commands_copy->cat));
-#endif
-
+            snprintf(buf, sizeof(buf), "%s commands:",
+                     get_category_name(cmd->cat));
+            LINE(line->str);
             H2(buf);
-            current_cat = commands_copy->cat;
-        }
 
-#ifdef CLIENTHELP_LONG_LIST
-        if (commands_copy->desc != NULL) {
-            char buf[MAX_BUF];
-            snprintf(buf, MAX_BUF - 1, "%s - %s", commands_copy->name, commands_copy->desc);
-            LINE(buf);
-        } else {
-            LINE(commands_copy->name);
+            category = cmd->cat;
+            g_string_free(line, true);
+            line = g_string_new(NULL);
         }
-#else
-        {
-            size_t name_len;
-
-            name_len = strlen(commands_copy->name);
-
-            if (strlen(commands_copy->name) > MAX_BUF) {
-                LINE(commands_copy->name);
-            } else if (name_len > assumed_wrap) {
-                LINE(line_buf);
-                LINE(commands_copy->name);
-                line_len = 0;
-            } else if (line_len + name_len > assumed_wrap) {
-                LINE(line_buf);
-                strncpy(line_buf, commands_copy->name, name_len + 1);
-                line_len = name_len;
-            } else {
-                if (line_len > 0) {
-                    strncat(line_buf, " ", 2);
-                    line_len += 1;
-                }
-                strncat(line_buf, commands_copy->name, name_len + 1);
-                line_len += name_len;
-            }
-        }
-#endif
+        g_string_append_printf(line, "%s ", cmd->name);
     }
 
-#ifndef CLIENTHELP_LONG_LIST
-    if (line_len) {
-        LINE(line_buf);
-    }
-#endif
+    LINE(line->str);
+    g_string_free(line, true);
 }
 
-/**
- *
- * @param cc
- */
 static void show_help(const ConsoleCommand *cc) {
     char buf[MAX_BUF];
-
     if (cc->desc != NULL) {
         snprintf(buf, MAX_BUF - 1, "%s - %s", cc->name, cc->desc);
-        H2(buf);
     } else {
         snprintf(buf, MAX_BUF - 1, "Help for '%s':", cc->name);
-        H2(buf);
     }
+    H2(buf);
 
     if (cc->helpfunc != NULL) {
         const char *long_help = NULL;
@@ -180,94 +86,7 @@ static void show_help(const ConsoleCommand *cc) {
     }
 }
 
-/**
- *
- * @param arg
- */
-static void do_clienthelp(const char * arg)
-{
-    const ConsoleCommand * cc;
-
-    if (!arg || !strlen(arg)) {
-        do_clienthelp_list();
-        return;
-    }
-
-    cc = find_command(arg);
-
-    if (cc == NULL) {
-        char buf[MAX_BUF];
-        snprintf(buf, MAX_BUF - 1, "clienthelp: Unknown command %s.", arg);
-        draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE, buf);
-        return;
-    }
-
-    show_help(cc);
-
-}
-
-/**
- *
- */
-static const char * help_clienthelp(void)
-{
-    return
-        "Syntax:\n"
-        "\n"
-        "    clienthelp\n"
-        "    clienthelp <command>\n"
-        "\n"
-        "Without any arguments, displays a list of client-side "
-        "commands.\n"
-        "\n"
-        "With arguments, displays the help for the client-side "
-        "command <command>.\n"
-        "\n"
-        "See also: serverhelp, help.";
-}
-
-/**
- *
- * @param arg
- */
-static void do_serverhelp(const char * arg)
-{
-
-    if (arg) {
-        char buf[MAX_BUF];
-        snprintf(buf, sizeof(buf), "help %s", arg);
-        /* maybe not a must send, but we probably don't want to drop it */
-        send_command(buf, -1, 1);
-    } else {
-        send_command("help", -1, 1); /* TODO make install in server branch doesn't install def_help. */
-    }
-}
-
-/**
- *
- */
-static const char * help_serverhelp(void)
-{
-    return
-        "Syntax:\n"
-        "\n"
-        "    serverhelp\n"
-        "    serverhelp <command>\n"
-        "\n"
-        "Fetches help from the server.\n"
-        "\n"
-        "Note that currently nothing can be done (without a recompile) if a "
-        "client command masks a server command.\n"
-        "\n"
-        "See also: clienthelp, help.";
-}
-
-/**
- *
- * @param cpnext
- */
-static void command_help(const char *cpnext)
-{
+static void command_help(const char *cpnext) {
     if (cpnext) {
         const ConsoleCommand * cc;
         char buf[MAX_BUF];
@@ -291,11 +110,7 @@ static void command_help(const char *cpnext)
     }
 }
 
-/**
- *
- */
-static const char * help_help(void)
-{
+static const char * help_help(void) {
     return
         "Syntax:\n"
         "\n"
@@ -308,25 +123,14 @@ static const char * help_help(void)
         "\n"
         "With arguments, first checks if there's a client command "
         "named <topic>. If there is, display it's help. If there "
-        "isn't, send the topic to the server.\n"
-        "\n"
-        "See also: clienthelp, serverhelp.";
+        "isn't, send the topic to the server.";
 }
 
 /**
  * @} */ /* EndOf PCmdHelpCommands
  */
 
-/*
- * Other commands.
- */
-
-/**
- *
- * @param cpnext
- */
-static void set_command_window(const char *cpnext)
-{
+static void set_command_window(const char *cpnext) {
     if (!cpnext) {
         draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
                       "cwindow command requires a number parameter");
@@ -340,31 +144,20 @@ static void set_command_window(const char *cpnext)
     }
 }
 
-/**
- *
- * @param cpnext
- */
-static void command_foodbeep(const char *cpnext)
-{
-    (void)cpnext; /* __UNUSED__ */
+static void command_foodbeep() {
     if (want_config[CONFIG_FOODBEEP]) {
-        want_config[CONFIG_FOODBEEP]=0;
+        want_config[CONFIG_FOODBEEP] = 0;
         draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
                       "Warning bell when low on food disabled");
     } else {
-        want_config[CONFIG_FOODBEEP]=1;
+        want_config[CONFIG_FOODBEEP] = 1;
         draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
                       "Warning bell when low on food enabled");
     }
     use_config[CONFIG_FOODBEEP] = want_config[CONFIG_FOODBEEP];
 }
 
-/**
- *
- * @param cat
- */
-const char * get_category_name(CommCat cat)
-{
+const char * get_category_name(CommCat cat) {
     const char * cat_name;
 
     /* HACK Need to keep this in sync. with player.h */
@@ -372,9 +165,6 @@ const char * get_category_name(CommCat cat)
     case COMM_CAT_MISC:
         cat_name = "Miscellaneous";
         break;
-    case COMM_CAT_HELP:
-        cat_name = "Help";
-        break;
     case COMM_CAT_INFO:
         cat_name = "Informational";
         break;
@@ -401,167 +191,52 @@ const char * get_category_name(CommCat cat)
  * Implementation basically stolen verbatim from the server.
  */
 
-/* "Typecasters" (and some forwards) */
+static void do_script_list() { script_list(); }
 
-/**
- *
- * @param ignored
- */
-static void do_script_list(const char * ignored)
-{
-    script_list();
-}
+static void do_clearinfo() { menu_clear(); }
 
-/**
- *
- * @param ignored
- */
-static void do_clearinfo(const char * ignored)
-{
-    menu_clear();
-}
+static void do_disconnect() { client_disconnect(); }
 
-/**
- *
- * @param ignored
- */
-static void do_disconnect(const char * ignored)
-{
-    close_server_connection();
 
-    /* the gtk clients need to do some cleanup logic - otherwise,
-     * they start hogging CPU.
-     */
-    cleanup_connection();
-    return;
-}
+static void do_inv() { print_inventory(cpl.ob); }
 
-#ifdef HAVE_DMALLOC_H
-#ifndef DMALLOC_VERIFY_NOERROR
-#define DMALLOC_VERIFY_NOERROR  1
-#endif
-/**
- *
- * @param ignored
- */
-static void do_dmalloc(const char * ignored)
-{
-    if (dmalloc_verify(NULL)==DMALLOC_VERIFY_NOERROR)
-        draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
-                      "Heap checks out OK");
-    else
-        draw_ext_info(NDI_RED, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_ERROR,
-                      "Heap corruption detected");
-}
-#endif
-
-/**
- *
- * @param ignored
- */
-static void do_inv(const char * ignored)
-{
-    print_inventory (cpl.ob);
-}
-
-static void do_magicmap(const char * ignored)
-{
-    cpl.showmagic=1;
+static void do_magicmap() {
     draw_magic_map();
 }
 
-/**
- *
- * @param ignored
- */
-static void do_metaserver(const char * ignored)
-{
-    if (!metaserver_get_info(meta_server, meta_port)) {
-        metaserver_show(FALSE);
-    } else
-        draw_ext_info(
-            NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_METASERVER,
-            "Unable to get metaserver information.");
-}
+static void do_savedefaults() { save_defaults(); }
 
-/**
- *
- * @param ignored
- */
-static void do_savedefaults(const char * ignored)
-{
-    save_defaults();
-}
+static void do_savewinpos() { save_winpos(); }
 
-/**
- *
- * @param ignored
- */
-static void do_savewinpos(const char * ignored)
-{
-    save_winpos();
-}
-
-/**
- *
- * @param used
- */
-static void do_take(const char * used)
-{
+static void do_take(const char *used) {
     command_take("take", used); /* I dunno why they want it. */
 }
 
-/**
- *
- * @param ignored
- */
-static void do_num_free_items(const char * ignored)
-{
-    LOG(LOG_INFO,"common::extended_command","num_free_items=%d", num_free_items());
+static void do_num_free_items() {
+    LOG(LOG_INFO, "common::extended_command", "num_free_items=%d",
+        num_free_items());
 }
 
-static void do_clienthelp(const char * arg); /* Forward. */
-
 /* Help "typecasters". */
 #include "chelp.h"
 
-/**
- *
- */
-static const char * help_bind(void)
-{
+static const char * help_bind(void) {
     return HELP_BIND_LONG;
 }
 
-/**
- *
- */
-static const char * help_unbind(void)
-{
+static const char * help_unbind(void) {
     return HELP_UNBIND_LONG;
 }
 
-/**
- *
- */
-static const char * help_magicmap(void)
-{
+static const char * help_magicmap(void) {
     return HELP_MAGICMAP_LONG;
 }
 
-/**
- *
- */
-static const char * help_inv(void)
-{
+static const char * help_inv(void) {
     return HELP_INV_LONG;
 }
 
-/**
- *
- */
-static const char * help_cwindow(void)
-{
+static const char * help_cwindow(void) {
     return
         "Syntax:\n"
         "\n"
@@ -574,9 +249,6 @@ static const char * help_cwindow(void)
         "(What does this mean, 'put a lid on it'?) TODO";
 }
 
-/**
- *
- */
 static const char * help_script(void) {
     return
         "Syntax: script <path>\n\n"
@@ -584,11 +256,7 @@ static const char * help_script(void) {
         "client-side scripting, please see the Crossfire Wiki.";
 }
 
-/**
- *
- */
-static const char * help_scripttell(void)
-{
+static const char * help_scripttell(void) {
     return
         "Syntax:\n"
         "\n"
@@ -599,11 +267,7 @@ static const char * help_scripttell(void)
 
 /* Toolkit-dependent. */
 
-/**
- *
- */
-static const char * help_savewinpos(void)
-{
+static const char * help_savewinpos(void) {
     return
         "Syntax:\n"
         "\n"
@@ -612,32 +276,7 @@ static const char * help_savewinpos(void)
         "save window positions - split windows mode only.";
 }
 
-/**
- *
- */
-static const char * help_metaserver(void)
-{
-    /* TODO Add command_escape() where appropriate. On the other
-    hand, that can lead to a meaningless syntax-display API.*/
-
-    return
-        "Syntax:\n"
-        "\n"
-        "    metaserver\n"
-        "\n"
-        "Get updated list of servers "
-        "from the metaserver and show it."
-        "This is the same information that the client "
-        "uses to show a list of servers when it starts.\n"
-        "\n"
-        "Warning: This command may freeze the client until it gets the list.";
-}
-
-/**
- *
- */
-static const char * help_scriptkill(void)
-{
+static const char * help_scriptkill(void) {
     return
         "Syntax:\n"
         "\n"
@@ -647,274 +286,74 @@ static const char * help_scriptkill(void)
         "(Not guaranteed to work?)";
 }
 
-/**
- *
- */
-static const char * help_showweight(void)
-{
-    return
-        "Syntax:\n"
-        "\n"
-        "    showweight\n"
-        "    showweight inventory\n"
-        "    showweight look\n"
-        "\n"
-        "(Or any prefix of the arguments.)"
-        "Toggles if you see the weight of"
-        "items in your inventory (also if"
-        "no argument given) or your"
-        "look-window.";
+static void cmd_raw(const char *cmd) {
+    cs_print_string(csocket.fd, "%s", cmd);
 }
 
-/*
-*	draw_ext_info(NDI_NAVY, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
-            "Information Commands");*
-	draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
-            " inv         - *recursively* print your");
-	draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
-            "               inventory - includes containers.");
-	draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
-            " mapredraw, showinfo, take");
-	draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
-            " help        - show this message");
-	draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
-            " help <command> - get more information on a");
-	draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
-            "                command (Server command only?)");
-	draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
-            " showicon    - draw status icons in");
-	draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
-            "               inventory window");
-	draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
-            " showweight  - show weight in inventory");
-	draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
-            "               look windows");
-	draw_ext_info(NDI_NAVY, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
-            "Scripting Commands");
-	draw_ext_info(NDI_NAVY, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
-            "Client Side Debugging Commands");
-#ifdef HAVE_DMALLOC_H
-	draw_ext_info(NDI_BLACK, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE,
-            " dmalloc     - Check heap?");
-#endif
-*/
-
-/* TODO Wrap these? Um. */
 static ConsoleCommand CommonCommands[] = {
-    /* From player.h:
-        name, cat,
-        func, helpfunc,
-        long_desc
-    */
+    {"cmd", COMM_CAT_DEBUG, cmd_raw, NULL, "Send a raw command to the server"},
 
-    {
-        "autorepeat", COMM_CAT_MISC,
-        set_autorepeat, NULL,
-        "toggle autorepeat" /* XXX Eh? */
-    },
+    {"bind", COMM_CAT_SETUP, bind_key, help_bind, HELP_BIND_SHORT},
 
-    {
-        "bind", COMM_CAT_SETUP,
-        bind_key, help_bind,
-        HELP_BIND_SHORT
-    },
-
-    {
-        "script", COMM_CAT_SCRIPT,
-        script_init, help_script,
-        NULL
-    },
+    {"script", COMM_CAT_SCRIPT, script_init, help_script, NULL},
 #ifdef HAVE_LUA
-    {
-        "lua_load", COMM_CAT_SCRIPT,
-        script_lua_load, NULL, NULL
-    },
+    {"lua_load", COMM_CAT_SCRIPT, script_lua_load, NULL, NULL},
 
-    {
-        "lua_list", COMM_CAT_SCRIPT,
-        script_lua_list, NULL, NULL
-    },
+    {"lua_list", COMM_CAT_SCRIPT, script_lua_list, NULL, NULL},
 
-    {
-        "lua_kill", COMM_CAT_SCRIPT,
-        script_lua_kill, NULL, NULL
-    },
+    {"lua_kill", COMM_CAT_SCRIPT, script_lua_kill, NULL, NULL},
 #endif
-    {
-        "scripts", COMM_CAT_SCRIPT,
-        do_script_list, NULL,
-        "List the running scripts(?)"
-    },
+    {"scripts", COMM_CAT_SCRIPT, do_script_list, NULL, "List running scripts"},
+
+    {"scriptkill", COMM_CAT_SCRIPT, script_kill, help_scriptkill, NULL},
+
+    {"scripttell", COMM_CAT_SCRIPT, script_tell, help_scripttell, NULL},
+
+    {"clearinfo", COMM_CAT_MISC, do_clearinfo, NULL, "Clear message window"},
+
+    {"cwindow", COMM_CAT_SETUP, set_command_window, help_cwindow, NULL},
+
+    {"disconnect", COMM_CAT_MISC, do_disconnect, NULL, NULL},
+
+
+    {"foodbeep", COMM_CAT_SETUP, command_foodbeep, NULL,
+     "toggle audible low on food warning"},
+
+    {"help", COMM_CAT_MISC, command_help, help_help, NULL},
+
+    {"inv", COMM_CAT_DEBUG, do_inv, help_inv, HELP_INV_SHORT},
+
+    {"magicmap", COMM_CAT_MISC, do_magicmap, help_magicmap,
+     HELP_MAGICMAP_SHORT},
+
+    {"savedefaults", COMM_CAT_SETUP, do_savedefaults, NULL,
+     HELP_SAVEDEFAULTS_SHORT},
 
     {
-        "scriptkill", COMM_CAT_SCRIPT,
-        script_kill, help_scriptkill,
-        NULL
+     "savewinpos", COMM_CAT_SETUP, do_savewinpos, help_savewinpos,
+     "Saves the position and sizes of windows." /* Panes? */
     },
 
-    {
-        "scripttell", COMM_CAT_SCRIPT,
-        script_tell, help_scripttell,
-        NULL
-    },
+    {"take", COMM_CAT_MISC, do_take, NULL, NULL},
 
-    {
-        "clearinfo", COMM_CAT_MISC,
-        do_clearinfo, NULL,
-        "clear the info window"
-    },
-
-    {
-        "cwindow", COMM_CAT_SETUP,
-        set_command_window, help_cwindow,
-        NULL
-    },
-
-    {
-        "disconnect", COMM_CAT_MISC,
-        do_disconnect, NULL,
-        "close connection to server"
-    },
-
-
-#ifdef HAVE_DMALLOC_H
-    {
-        "dmalloc", COMM_CAT_DEBUG,
-        do_dmalloc, NULL,
-        NULL
-    },
-#endif
-
-    {
-        "foodbeep", COMM_CAT_SETUP,
-        command_foodbeep, NULL,
-        "toggle audible low on food warning"
-
-    },
-
-    {
-        "help", COMM_CAT_HELP,
-        command_help, help_help,
-        NULL
-    },
-
-    {
-        "clienthelp", COMM_CAT_HELP,
-        do_clienthelp, help_clienthelp,
-        "Client-side command information"
-    },
-
-    {
-        "serverhelp", COMM_CAT_HELP,
-        do_serverhelp, help_serverhelp,
-        "Server-side command information"
-    },
-
-    {
-        "inv", COMM_CAT_DEBUG,
-        do_inv, help_inv,
-        HELP_INV_SHORT
-    },
-
-    {
-        "magicmap", COMM_CAT_MISC,
-        do_magicmap, help_magicmap,
-        HELP_MAGICMAP_SHORT
-    },
-
-    {
-        "metaserver", COMM_CAT_INFO,
-        do_metaserver, help_metaserver,
-        "Print 'metaserver information'. Warning - your client will pause."
-    },
-
-    {
-        "savedefaults", COMM_CAT_SETUP,
-        do_savedefaults, NULL,
-        HELP_SAVEDEFAULTS_SHORT /* How do we make sure showicons stays on? */
-    },
-
-    {
-        "savewinpos", COMM_CAT_SETUP,
-        do_savewinpos, help_savewinpos,
-        "Saves the position and sizes of windows." /* Panes? */
-    },
-
-    {
-        "scroll", COMM_CAT_SETUP,
-        set_scroll, NULL,
-        "toggle scroll/wrap mode in info window"
-    },
-
-    {
-        "showicon", COMM_CAT_SETUP,
-        set_show_icon, NULL,
-        "Toggles if you see the worn, locked, cursed etc state in the inventory pane."
-    },
-
-    {
-        "showweight", COMM_CAT_SETUP,
-        set_show_weight, help_showweight,
-        "Toggles if you see item weights in inventory look windows."
-    },
-
-    {
-        "take", COMM_CAT_MISC,
-        do_take, NULL,
-        NULL
-    },
-
-    {
-        "unbind", COMM_CAT_SETUP,
-        unbind_key, help_unbind,
-        NULL
-    },
-
-    {
-        "num_free_items", COMM_CAT_DEBUG,
-        do_num_free_items, NULL,
-        "log the number of free items?"
-    },
-    {
-        "show", COMM_CAT_SETUP,
-        command_show, NULL,
-        "Change what items to show in inventory"
-    },
+    {"unbind", COMM_CAT_SETUP, unbind_key, help_unbind, NULL},
 
+    {"num_free_items", COMM_CAT_DEBUG, do_num_free_items, NULL,
+     "log the number of free items?"},
+    {"show", COMM_CAT_SETUP, command_show, NULL,
+     "Change what items to show in inventory"},
 };
 
-const int CommonCommandsSize = sizeof(CommonCommands) / sizeof(ConsoleCommand);
-
-#ifdef TOOLKIT_COMMANDS
-extern ConsoleCommand ToolkitCommands[];
-
-extern const int ToolkitCommandsSize;
-#endif
-
-/* ------------------------------------------------------------------ */
-
-int num_commands;
-
-/**
- *
- */
-int get_num_commands(void)
-{
+const size_t num_commands = sizeof(CommonCommands) / sizeof(ConsoleCommand);
+static int get_num_commands() {
     return num_commands;
 }
 
 static ConsoleCommand ** name_sorted_commands;
 
-/**
- *
- * @param a_
- * @param b_
- */
-static int sort_by_name(const void * a_, const void * b_)
-{
+static int sort_by_name(const void * a_, const void * b_) {
     ConsoleCommand * a = *((ConsoleCommand **)a_);
     ConsoleCommand * b = *((ConsoleCommand **)b_);
-
     return strcmp(a->name, b->name);
 }
 
@@ -922,13 +361,7 @@ static ConsoleCommand ** cat_sorted_commands;
 
 /* Sort by category, then by name. */
 
-/**
- *
- * @param a_
- * @param b_
- */
-static int sort_by_category(const void *a_, const void *b_)
-{
+static int sort_by_category(const void *a_, const void *b_) {
     /* Typecasts, so it goes. */
     ConsoleCommand * a = *((ConsoleCommand **)a_);
     ConsoleCommand * b = *((ConsoleCommand **)b_);
@@ -940,45 +373,19 @@ static int sort_by_category(const void *a_, const void *b_)
     return a->cat - b->cat;
 }
 
-/**
- *
- */
-void init_commands(void)
-{
-    int i;
-
-#ifdef TOOLKIT_COMMANDS
-    init_toolkit_commands();
-
-    /* TODO I dunno ... go through the list and print commands without helps? */
-    num_commands = CommonCommandsSize + ToolkitCommandsSize;
-#else
-    num_commands = CommonCommandsSize;
-#endif
-
-    /* Make a list of (pointers to statically allocated!) all the commands.
-       We have a list; the toolkit has a
-       ToolkitCommands and ToolkitCommandsSize, initialized before calling this.
-    */
-
+void init_commands() {
     /* XXX Leak! */
-    name_sorted_commands = malloc(sizeof(ConsoleCommand *) * num_commands);
+    name_sorted_commands = g_malloc(sizeof(ConsoleCommand *) * num_commands);
 
-    for (i = 0; i < CommonCommandsSize; i++) {
+    for (size_t i = 0; i < num_commands; i++) {
         name_sorted_commands[i] = &CommonCommands[i];
     }
 
-#ifdef TOOLKIT_COMMANDS
-    for(i = 0; i < ToolkitCommandsSize; i++) {
-        name_sorted_commands[CommonCommandsSize + i] = &ToolkitCommands[i];
-    }
-#endif
-
     /* Sort them. */
     qsort(name_sorted_commands, num_commands, sizeof(ConsoleCommand *), sort_by_name);
 
     /* Copy the list, then sort it by category. */
-    cat_sorted_commands = malloc(sizeof(ConsoleCommand *) * num_commands);
+    cat_sorted_commands = g_malloc(sizeof(ConsoleCommand *) * num_commands);
 
     memcpy(cat_sorted_commands, name_sorted_commands, sizeof(ConsoleCommand *) * num_commands);
 
@@ -987,21 +394,12 @@ void init_commands(void)
     /* TODO Add to the list of tab-completion items. */
 }
 
-#ifndef tolower
-#define tolower(C)      (((C) >= 'A' && (C) <= 'Z')? (C) - 'A' + 'a': (C))
-#endif
-
-/**
- *
- * @param cmd
- */
-const ConsoleCommand * find_command(const char * cmd)
-{
+const ConsoleCommand * find_command(const char *cmd) {
     ConsoleCommand ** asp_p = NULL, dummy;
     ConsoleCommand * dummy_p;
     ConsoleCommand * asp;
     char *cp, *cmd_cpy;
-    cmd_cpy = strdup(cmd);
+    cmd_cpy = g_strdup(cmd);
 
     for (cp=cmd_cpy; *cp; cp++) {
         *cp =tolower(*cp);
@@ -1036,8 +434,7 @@ const ConsoleCommand * find_command(const char * cmd)
  * Returns a pointer to the head of an array of ConsoleCommands sorted by
  * category, then by name.  It's num_commands long.
  */
-ConsoleCommand ** get_cat_sorted_commands(void)
-{
+ConsoleCommand ** get_cat_sorted_commands(void) {
     return cat_sorted_commands;
 }
 
@@ -1050,8 +447,7 @@ ConsoleCommand ** get_cat_sorted_commands(void)
  * @param cp
  * @param cpnext
  */
-int handle_local_command(const char* cp, const char * cpnext)
-{
+int handle_local_command(const char* cp, const char *cpnext) {
     const ConsoleCommand * cc = NULL;
 
     cc = find_command(cp);
@@ -1085,8 +481,7 @@ int handle_local_command(const char* cp, const char * cpnext)
  *
  * @param ocommand
  */
-void extended_command(const char *ocommand)
-{
+void extended_command(const char *ocommand) {
     const char *cp = ocommand;
     char *cpnext, command[MAX_BUF];
 
@@ -1106,6 +501,15 @@ void extended_command(const char *ocommand)
             cpnext = NULL;
         }
     }
+    /*
+     * Try to prevent potential client hang by trying to delete a
+     * character when there is no character to delete.
+     * Thus, only send quit command if there is a player to delete.
+     */
+    if (cpl.title[0] == '\0' && strcmp(cp, "quit") == 0){
+        // Bail here, there isn't anything this should be doing.
+        return;
+    }
 
     /* cp now contains the command (everything before first space),
      * and cpnext contains everything after that first space.  cpnext
@@ -1167,7 +571,7 @@ static const char *const commands[] = {
     "version", "wave", "weather", "west", "whereabouts", "whereami", "whistle",
     "who", "wimpy", "wink", "yawn",
 };
-#define NUM_COMMANDS ((int)(sizeof(commands) / sizeof(char*)))
+const size_t num_server_commands = sizeof(commands) / sizeof(char *);
 
 /**
  * Player has entered 'command' and hit tab to complete it.  See if we can
@@ -1176,9 +580,8 @@ static const char *const commands[] = {
  *
  * @param command
  */
-const char * complete_command(const char *command)
-{
-    int i, len, display;
+const char * complete_command(const char *command) {
+    int len, display = 0;
     const char *match;
     static char result[64];
     char list[500];
@@ -1189,7 +592,6 @@ const char * complete_command(const char *command)
         return NULL;
     }
 
-    display = 0;
     strcpy(list, "Matching commands:");
 
     /* TODO Partial match, e.g.:
@@ -1205,7 +607,7 @@ const char * complete_command(const char *command)
     match = NULL;
 
     /* check server side commands */
-    for (i=0; i<NUM_COMMANDS; i++) {
+    for (size_t i = 0; i < num_server_commands; i++) {
         if (!strncmp(command, commands[i], len)) {
             if (display) {
                 snprintf(list + strlen(list), 499 - strlen(list), " %s", commands[i]);
@@ -1220,7 +622,7 @@ const char * complete_command(const char *command)
     }
 
     /* check client side commands */
-    for (i=0; i<CommonCommandsSize; i++) {
+    for (size_t i = 0; i < num_commands; i++) {
         if (!strncmp(command, CommonCommands[i].name, len)) {
             if (display) {
                 snprintf(list + strlen(list), 499 - strlen(list), " %s", CommonCommands[i].name);
@@ -1251,8 +653,5 @@ const char * complete_command(const char *command)
      * the excess space should be stripped off automatically.
      */
     snprintf(result, sizeof(result), "%s ", match);
-
     return result;
 }
-
-#endif /* CPROTO */
