@@ -30,6 +30,8 @@
 #include "external.h"
 #include "script.h"
 
+bool profile_latency = false;
+
 /** Array for direction strings for each numeric direction. */
 const char *const directions[] = {"stay",      "north",     "northeast",
                                   "east",      "southeast", "south",
@@ -199,6 +201,10 @@ int send_command(const char *command, int repeat, int must_send) {
             SockList_AddInt(&sl, repeat);
             SockList_AddString(&sl, command);
             SockList_Send(&sl, csocket.fd);
+            if (profile_latency) {
+                printf("profile/com,%d,%" G_GINT64_FORMAT ",%s\n",
+                       csocket.command_sent, g_get_monotonic_time(), command);
+            }
         }
     } else {
         cs_print_string(csocket.fd, "command %d %s", repeat,command);
@@ -216,7 +222,13 @@ void CompleteCmd(unsigned char *data, int len) {
     }
     csocket.command_received = GetShort_String(data);
     csocket.command_time = GetInt_String(data+2);
-    script_sync(csocket.command_sent - csocket.command_received);
+    const int in_flight = csocket.command_sent - csocket.command_received;
+    if (profile_latency) {
+        gint64 t = g_get_monotonic_time();
+        printf("profile/comc,%d,%" G_GINT64_FORMAT ",%d,%d\n",
+               csocket.command_received, t, csocket.command_time, in_flight);
+    }
+    script_sync(in_flight);
 }
 
 /* This does special processing on the 'take' command.  If the
