@@ -32,6 +32,7 @@ extern void on_entry_account_password_activate(GtkEntry *entry, gpointer user_da
 extern void update_login_info(int type);
 extern void start_login(int method);
 extern void choose_char_window_show();
+extern void account_show_login(void);
 /* config.c */
 extern void load_theme(int reload);
 extern void config_check();
@@ -59,7 +60,7 @@ extern void image_update_download_status(int start, int end, int total);
 extern void get_map_image_size(int face, guint8 *w, guint8 *h);
 extern void init_image_cache_data(void);
 /* info.c */
-extern void set_text_tag_from_style(GtkTextTag *tag, GtkStyle *style, GtkStyle *base_style);
+extern void set_text_tag_from_style(GtkTextTag *tag, GtkStyle *style, const GtkStyle * const base_style);
 extern void add_tags_to_textbuffer(Info_Pane *pane, GtkTextBuffer *textbuf);
 extern void add_style_to_textbuffer(Info_Pane *pane, GtkStyle *base_style);
 extern void info_get_styles(void);
@@ -126,6 +127,7 @@ extern void error_dialog(char *description, char *information);
 extern void my_log_handler(const gchar *log_domain, GLogLevelFlags log_level, const gchar *message, gpointer user_data);
 extern int main(int argc, char *argv[]);
 extern void get_window_coord(GtkWidget *win, int *x, int *y, int *wx, int *wy, int *w, int *h);
+extern void show_main_client(void);
 /* map.c */
 extern void map_init(GtkWidget *window_root);
 extern int display_mapscroll(int dx, int dy);
