@@ -29,25 +29,15 @@
 
 #include "image.h"
 #include "main.h"
-#include "mapdata.h"
 #include "gtk2proto.h"
 
 extern GtkWidget *window_root; /**< In main.c */
 int image_size=DEFAULT_IMAGE_SIZE;
 
-struct {
-    char    *name;
-    guint32  checksum;
-    guint8   *png_data;
-    guint32  width, height;
-} private_cache[MAXPIXMAPNUM];
-
 #define BPP 4
 
 PixmapInfo *pixmaps[MAXPIXMAPNUM];
 
-int last_face_num=0;
-
 /* Do we have new images to display? */
 int have_new_image=0;
 
@@ -217,8 +207,8 @@ static void free_pixmap(PixmapInfo *pi)
     if (use_config[CONFIG_DISPLAYMODE]==CFG_DM_SDL) {
 #ifdef HAVE_SDL
         if (pi->map_image) {
+            g_free(((SDL_Surface*)pi->map_image)->pixels);
             SDL_FreeSurface(pi->map_image);
-            free(((SDL_Surface*)pi->map_image)->pixels);
             SDL_FreeSurface(pi->fog_image);
             /*
              * Minor memory leak here - SDL_FreeSurface() frees the pixel
@@ -529,9 +519,6 @@ void init_image_cache_data(void)
     GtkStyle *style;
 #include "../../pixmaps/question.xpm"
 
-
-    LOG(LOG_DEBUG, "gtk-v2::init_image_cache_data", "Init Image Cache");
-
     style = gtk_widget_get_style(window_root);
     pixmaps[0] = g_new(PixmapInfo, 1);
     pixmaps[0]->icon_image =
