@@ -53,9 +53,10 @@ static double weight_limit;
  * complicated because position of elements within the array are important, so
  * a simple g_realloc won't work.
  */
-#define MAX_INV_COLUMNS 20
-#define MAX_INV_ROWS    100
-static GtkWidget *inv_table_children[MAX_INV_ROWS][MAX_INV_COLUMNS];
+#define MAX_INV         2000
+static GtkWidget *inv_table_children[MAX_INV];
+
+#define INV_TABLE_AT(x, y, cols) inv_table_children[cols*y + x]
 
 /* Different styles we recognize */
 enum Styles {
@@ -99,13 +100,11 @@ typedef struct {
                                  */
     enum display_type type; /**< Type of widget */
     GtkWidget *treeview; /**< treeview widget for this tab */
-    GtkTreeStore *treestore; /**< store of data for treeview */
 } Notebook_Info;
 
-/* Prototypes for static functions */
-static gboolean on_inv_table_expose_event(GtkWidget *widget,
-        GdkEventExpose *event, gpointer user_data);
+static GtkTreeStore *treestore; /**< store of data for treeview */
 
+/* Prototypes for static functions */
 static void on_switch_page(GtkNotebook *notebook, gpointer *page,
                            guint page_num, gpointer user_data);
 
@@ -468,12 +467,22 @@ void inventory_init(GtkWidget *window_root) {
 
     g_signal_connect((gpointer)inv_notebook, "switch_page",
                      (GCallback)on_switch_page, NULL);
-    g_signal_connect((gpointer) inv_table, "expose_event",
-            (GCallback) on_inv_table_expose_event, NULL);
     g_signal_connect((gpointer) treeview_look, "row_collapsed",
             (GCallback) list_row_collapse, NULL);
 
-    memset(inv_table_children, 0, sizeof (GtkWidget *) * MAX_INV_ROWS * MAX_INV_COLUMNS);
+    memset(inv_table_children, 0, sizeof (GtkWidget *) * MAX_INV);
+
+    treestore = gtk_tree_store_new(LIST_NUM_COLUMNS,
+            G_TYPE_STRING,
+            G_TYPE_OBJECT,
+            G_TYPE_STRING,
+            G_TYPE_STRING,
+            G_TYPE_POINTER,
+            GDK_TYPE_COLOR,
+            G_TYPE_INT,
+            G_TYPE_STRING,
+            GDK_TYPE_COLOR,
+            PANGO_TYPE_FONT_DESCRIPTION);
 
     store_look = gtk_tree_store_new(LIST_NUM_COLUMNS,
             G_TYPE_STRING,
@@ -526,20 +535,8 @@ void inventory_init(GtkWidget *window_root) {
 
             gtk_notebook_insert_page(GTK_NOTEBOOK(inv_notebook), swindow, image, i);
 
-            inv_notebooks[i].treestore = gtk_tree_store_new(LIST_NUM_COLUMNS,
-                    G_TYPE_STRING,
-                    G_TYPE_OBJECT,
-                    G_TYPE_STRING,
-                    G_TYPE_STRING,
-                    G_TYPE_POINTER,
-                    GDK_TYPE_COLOR,
-                    G_TYPE_INT,
-                    G_TYPE_STRING,
-                    GDK_TYPE_COLOR,
-                    PANGO_TYPE_FONT_DESCRIPTION);
-
             inv_notebooks[i].treeview = gtk_tree_view_new_with_model(GTK_TREE_MODEL(
-                    inv_notebooks[i].treestore));
+                    treestore));
 
             g_signal_connect((gpointer) inv_notebooks[i].treeview, "row_collapsed",
                     G_CALLBACK(list_row_collapse), NULL);
@@ -572,7 +569,6 @@ void inventory_init(GtkWidget *window_root) {
  * @param op
  */
 void close_container(item *op) {
-    draw_lists();
 }
 
 /**
@@ -580,7 +576,6 @@ void close_container(item *op) {
  * @param op
  */
 void open_container(item *op) {
-    draw_lists();
 }
 
 /**
@@ -783,7 +778,7 @@ static void draw_inv_list(int tab) {
      * changed.  As such, we are forced to basicly redraw the entire list each
      * time this is called.
      */
-    gtk_tree_store_clear(inv_notebooks[tab].treestore);
+    gtk_tree_store_clear(treestore);
 
     for (tmp = cpl.ob->inv; tmp; tmp = tmp->next) {
         rowflag = inv_notebooks[tab].show_func(tmp);
@@ -791,7 +786,7 @@ static void draw_inv_list(int tab) {
             continue;
         }
 
-        add_object_to_store(tmp, inv_notebooks[tab].treestore, &iter, NULL, rowflag & INV_SHOW_COLOR);
+        add_object_to_store(tmp, treestore, &iter, NULL, rowflag & INV_SHOW_COLOR);
 
         if ((cpl.container == tmp) && tmp->open) {
             item *tmp2;
@@ -814,10 +809,10 @@ static void draw_inv_list(int tab) {
                 if (!(rowflag & INV_SHOW_ITEM)) {
                     continue;
                 }
-                add_object_to_store(tmp2, inv_notebooks[tab].treestore, &iter1, &iter,
+                add_object_to_store(tmp2, treestore, &iter1, &iter,
                         rowflag & INV_SHOW_COLOR);
             }
-            path = gtk_tree_model_get_path(GTK_TREE_MODEL(inv_notebooks[tab].treestore), &iter);
+            path = gtk_tree_model_get_path(GTK_TREE_MODEL(treestore), &iter);
             gtk_tree_view_expand_row(GTK_TREE_VIEW(inv_notebooks[tab].treeview), path, FALSE);
             gtk_tree_path_free(path);
         }
@@ -888,13 +883,15 @@ static void draw_inv_table(int animate) {
         num_items++;
     }
 
+    if (num_items > MAX_INV) {
+        LOG(LOG_ERROR, "draw_inv_table", "Too many items in inventory!");
+        return;
+    }
+
     GtkAllocation size;
     gtk_widget_get_allocation(inv_table, &size);
 
     columns = size.width / image_size;
-    if (columns > MAX_INV_COLUMNS) {
-        columns = MAX_INV_COLUMNS;
-    }
     rows = size.height / image_size;
 
     if (num_items > columns * rows) {
@@ -903,21 +900,18 @@ static void draw_inv_table(int animate) {
             rows++;
         }
     }
-    if (rows > MAX_INV_ROWS) {
-        rows = MAX_INV_ROWS;
-    }
 
     gtk_table_resize(GTK_TABLE(inv_table), rows, columns);
 
     x = 0;
     y = 0;
     for (tmp = cpl.ob->inv; tmp; tmp = tmp->next) {
-        if (inv_table_children[x][y] == NULL) {
-            inv_table_children[x][y] = gtk_drawing_area_new();
-            gtk_widget_set_size_request(inv_table_children[x][y], image_size,
+        if (INV_TABLE_AT(x, y, columns) == NULL) {
+            INV_TABLE_AT(x, y, columns) = gtk_drawing_area_new();
+            gtk_widget_set_size_request(INV_TABLE_AT(x, y, columns), image_size,
                                         image_size);
 
-            gtk_table_attach(GTK_TABLE(inv_table), inv_table_children[x][y],
+            gtk_table_attach(GTK_TABLE(inv_table), INV_TABLE_AT(x, y, columns),
                     x, x + 1, y, y + 1, GTK_FILL, GTK_FILL, 0, 0);
         }
         if (animate) {
@@ -935,7 +929,7 @@ static void draw_inv_table(int animate) {
                     tmp->last_anim = 0;
 
                     draw_inv_table_icon(
-                        gtk_widget_get_window(inv_table_children[x][y]),
+                        gtk_widget_get_window(INV_TABLE_AT(x, y, columns)),
                         pixmaps[tmp->face]->icon_image);
                 }
             }
@@ -946,58 +940,58 @@ static void draw_inv_table(int animate) {
              * effectively stacked - you can have 6 signal handlers tied to the
              * same function.
              */
-            handler = g_signal_handler_find((gpointer) inv_table_children[x][y],
+            handler = g_signal_handler_find((gpointer) INV_TABLE_AT(x, y, columns),
                     G_SIGNAL_MATCH_FUNC, 0, 0, NULL,
                     G_CALLBACK(drawingarea_inventory_table_button_press_event),
                     NULL);
 
             if (handler) {
-                g_signal_handler_disconnect((gpointer) inv_table_children[x][y], handler);
+                g_signal_handler_disconnect((gpointer) INV_TABLE_AT(x, y, columns), handler);
             }
 
-            handler = g_signal_handler_find((gpointer) inv_table_children[x][y],
+            handler = g_signal_handler_find((gpointer) INV_TABLE_AT(x, y, columns),
                     G_SIGNAL_MATCH_FUNC, 0, 0, NULL,
                     G_CALLBACK(drawingarea_inventory_table_expose_event),
                     NULL);
             if (handler) {
-                g_signal_handler_disconnect((gpointer) inv_table_children[x][y], handler);
+                g_signal_handler_disconnect((gpointer) INV_TABLE_AT(x, y, columns), handler);
             }
             /*
              * Not positive precisely what events are needed, but some events
              * beyond just the button press are necessary for the tooltips to
              * work.
              */
-            gtk_widget_add_events(inv_table_children[x][y], GDK_ALL_EVENTS_MASK);
+            gtk_widget_add_events(INV_TABLE_AT(x, y, columns), GDK_ALL_EVENTS_MASK);
 
-            g_signal_connect((gpointer) inv_table_children[x][y], "button_press_event",
+            g_signal_connect((gpointer) INV_TABLE_AT(x, y, columns), "button_press_event",
                     G_CALLBACK(drawingarea_inventory_table_button_press_event),
                     tmp);
 
-            g_signal_connect((gpointer) inv_table_children[x][y], "expose_event",
+            g_signal_connect((gpointer) INV_TABLE_AT(x, y, columns), "expose_event",
                     G_CALLBACK(drawingarea_inventory_table_expose_event),
                     tmp);
 
             /* Draw the inventory icon image to the table. */
-            draw_inv_table_icon(gtk_widget_get_window(inv_table_children[x][y]),
+            draw_inv_table_icon(gtk_widget_get_window(INV_TABLE_AT(x, y, columns)),
                                 pixmaps[tmp->face]->icon_image);
 
             // Draw an extra indicator if the item is applied.
             if (tmp->applied) {
-                gtk_widget_modify_bg(inv_table_children[x][y],
+                gtk_widget_modify_bg(INV_TABLE_AT(x, y, columns),
                         GTK_STATE_NORMAL, &applied_color);
             } else {
-                gtk_widget_modify_bg(inv_table_children[x][y],
+                gtk_widget_modify_bg(INV_TABLE_AT(x, y, columns),
                         GTK_STATE_NORMAL, NULL);
             }
 
-            gtk_widget_show(inv_table_children[x][y]);
+            gtk_widget_show(INV_TABLE_AT(x, y, columns));
             /*
              * Use tooltips to provide additional detail about the icons.
              * Looking at the code, the tooltip widget will take care of
              * removing the old tooltip, freeing strings, etc.
              */
             snprintf(buf, 255, "%s %s", tmp->d_name, tmp->flags);
-            gtk_widget_set_tooltip_text(inv_table_children[x][y], buf);
+            gtk_widget_set_tooltip_text(INV_TABLE_AT(x, y, columns), buf);
         }
         x++;
         if (x == columns) {
@@ -1015,28 +1009,9 @@ static void draw_inv_table(int animate) {
      * otherwise, we get errors on objects that are drawn.
      */
     for (i = num_items; i <= max_drawn; i++) {
-        if (inv_table_children[x][y]) {
-            gdk_window_clear(gtk_widget_get_window(inv_table_children[x][y]));
-
-            handler = g_signal_handler_find((gpointer) inv_table_children[x][y],
-                    G_SIGNAL_MATCH_FUNC, 0, 0, NULL,
-                    G_CALLBACK(drawingarea_inventory_table_button_press_event),
-                    NULL);
-
-            if (handler) {
-                g_signal_handler_disconnect((gpointer) inv_table_children[x][y], handler);
-            }
-
-            handler = g_signal_handler_find((gpointer) inv_table_children[x][y],
-                    G_SIGNAL_MATCH_FUNC, 0, 0, NULL,
-                    G_CALLBACK(drawingarea_inventory_table_expose_event),
-                    NULL);
-            if (handler) {
-                g_signal_handler_disconnect((gpointer) inv_table_children[x][y], handler);
-            }
-
-            /* Hide the widget so that the tooltips doesn't show up */
-            gtk_widget_hide(inv_table_children[x][y]);
+        if (INV_TABLE_AT(x, y, columns)) {
+            gtk_widget_destroy(INV_TABLE_AT(x, y, columns));
+            INV_TABLE_AT(x, y, columns) = NULL;
         }
         x++;
         if (x == columns) {
@@ -1112,24 +1087,11 @@ static void on_switch_page(GtkNotebook *notebook, gpointer *page,
 
     oldpage = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
     if (oldpage != page_num && inv_notebooks[oldpage].type == INV_TREE) {
-        gtk_tree_store_clear(inv_notebooks[oldpage].treestore);
+        gtk_tree_store_clear(treestore);
     }
     cpl.ob->inv_updated = 1;
 }
 
-/**
- *
- * @param widget
- * @param event
- * @param user_data
- * @return TRUE
- */
-static gboolean on_inv_table_expose_event(GtkWidget *widget,
-        GdkEventExpose *event, gpointer user_data) {
-    draw_inv_table(0);
-    return TRUE;
-}
-
 /**
  *
  */
@@ -1139,25 +1101,6 @@ static void animate_inventory() {
     item *tmp;
     int page;
     GtkTreeStore *store;
-    static int inv_tick = 0;
-
-    /*
-     * If global tick is set, then we are getting tick events from server to
-     * keep in sync, so we don't need the logic below.
-     */
-    if (!tick) {
-        /*
-         * The gtk client timeout is 12 times faster than that of the server so
-         * we slow it down here.  If we were really clever, we'd find what the
-         * timeout on the server actually is, and do gettimeofday calls here to
-         * remain very closely in sync.
-         */
-        inv_tick++;
-        if (inv_tick < 12) {
-            return;
-        }
-        inv_tick = 0;
-    }
 
     page = gtk_notebook_get_current_page(GTK_NOTEBOOK(inv_notebook));
 
@@ -1167,7 +1110,7 @@ static void animate_inventory() {
         return;
     }
 
-    store = inv_notebooks[page].treestore;
+    store = treestore;
 
     /* Get the first iter in the list */
     valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
@@ -1208,25 +1151,6 @@ static void animate_look() {
     gboolean valid;
     GtkTreeIter iter;
     item *tmp;
-    static int inv_tick = 0;
-
-    /*
-     * If global tick is set, then we are getting tick events from server to
-     * keep in sync, so we don't need the logic below.
-     */
-    if (!tick) {
-        /*
-         * The gtk client timeout is 12 times faster than that of the server so
-         * we slow it down here.  If we were really clever, we'd find what the
-         * timeout on the server actually is, and do gettimeofday calls here to
-         * remain very closely in sync.
-         */
-        inv_tick++;
-        if (inv_tick < 12) {
-            return;
-        }
-        inv_tick = 0;
-    }
 
     /* Get the first iter in the list */
     valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store_look), &iter);
