@@ -54,12 +54,18 @@ static const char *const colorname[NUM_COLORS] = {
 static gboolean updatekeycodes = FALSE;
 
 /* TODO: Move these declarations to actual header files. */
-extern int time_map_redraw;
+extern bool time_map_redraw;
+extern bool profile_latency;
 extern int MINLOG;
 extern SoundServer* server;
+bool debug_protocol;
+
+static char *connect_server = NULL;
 
 /** Command line options, descriptions, and parameters. */
 static GOptionEntry options[] = {
+    { "server", 's', 0, G_OPTION_ARG_STRING, &connect_server,
+        "Connect to the given server", "SERVER[:PORT]" },
     { "cache", 0, 0, G_OPTION_ARG_NONE, &want_config[CONFIG_CACHE],
         "Cache images", NULL },
     { "prefetch", 0, 0, G_OPTION_ARG_NONE, &want_config[CONFIG_DOWNLOAD],
@@ -72,10 +78,14 @@ static GOptionEntry options[] = {
     { "updatekeycodes", 0, 0, G_OPTION_ARG_NONE, &updatekeycodes,
         "Update the saved bindings for this keyboard", NULL },
 
-    { "time-redraw", 't', 0, G_OPTION_ARG_NONE, &time_map_redraw,
+    { "profile-latency", 0, 0, G_OPTION_ARG_NONE, &profile_latency,
+        "Log command acknowledgement latency to stdout", NULL },
+    { "profile-redraw", 0, 0, G_OPTION_ARG_NONE, &time_map_redraw,
         "Print map redraw times to stdout", NULL },
     { "verbose", 'v', 0, G_OPTION_ARG_INT, &MINLOG,
         "Set verbosity (0 is the most verbose)", "LEVEL" },
+    { "debug-protocol", 0, 0, G_OPTION_ARG_NONE, &debug_protocol,
+        "Print commands to and from the server", NULL },
     { NULL }
 };
 
@@ -84,10 +94,8 @@ char window_xml_file[MAX_BUF];
 GdkColor root_color[NUM_COLORS];
 
 GtkBuilder *dialog_xml, *window_xml;
-GtkWidget *window_root, *magic_map;
-
-/** Track whether the client has received a trick since the last redraw. */
-bool next_tick = false;
+GtkWidget *window_root, *magic_map, *connect_window;
+GtkNotebook *main_notebook;
 
 #ifdef WIN32 /* Win32 scripting support */
 static int do_scriptout() {
@@ -97,10 +105,29 @@ static int do_scriptout() {
 #endif /* WIN32 */
 
 /**
- * Map, spell, and inventory maintenance.
- * @return TRUE
+ * Redraw the map. Do a full redraw if there are new images to show. Return
+ * false to unregister this event source after one redraw.
  */
-static gboolean do_timeout(gpointer data) {
+static gboolean redraw(gpointer data) {
+    if (have_new_image) {
+        if (cpl.container) {
+            cpl.container->inv_updated = 1;
+        }
+        cpl.ob->inv_updated = 1;
+
+        have_new_image = 0;
+        draw_map(1);
+    } else {
+        draw_map(0);
+    }
+    draw_lists();
+    return FALSE;
+}
+
+/**
+ * Called whenever the server sends a tick command.
+ */
+void client_tick(guint32 tick) {
     if (cpl.showmagic) {
         if (gtk_notebook_get_current_page(GTK_NOTEBOOK(map_notebook)) !=
             MAGIC_MAP_PAGE) {
@@ -114,50 +141,11 @@ static gboolean do_timeout(gpointer data) {
     if (cpl.spells_updated) {
         update_spell_information();
     }
-    if (!tick) {
-        inventory_tick();
-        mapdata_animation();
-    }
 
-    return TRUE;
-}
-
-/**
- * Redraw the map. Do a full redraw if there are new images to show.
- */
-static gboolean redraw(gpointer data) {
-    // Do not redraw if no tick has arrived since the last redraw.
-    if (next_tick) {
-        next_tick = false;
-    } else {
-        // Sleep for 10 ms to prevent busy wait.
-        g_usleep(10 * 1e3);
-        return TRUE;
-    }
-
-    if (have_new_image) {
-        if (cpl.container) {
-            cpl.container->inv_updated = 1;
-        }
-        cpl.ob->inv_updated = 1;
-
-        have_new_image = 0;
-        draw_map(1);
-        draw_lists();
-    } else {
-        draw_map(0);
-    }
-    return TRUE;
-}
-
-/**
- * X11 client doesn't care about this
- */
-void client_tick(guint32 tick) {
     info_buffer_tick();
     inventory_tick();
     mapdata_animation();
-    next_tick = true;
+    g_idle_add(redraw, NULL);
 }
 /**
  * Handles client shutdown.
@@ -193,7 +181,6 @@ static gboolean do_network(GObject *stream, gpointer data) {
         script_process(&tmp_read);
     }
 
-    draw_lists();
     return TRUE;
 }
 
@@ -201,21 +188,21 @@ static gboolean do_network(GObject *stream, gpointer data) {
  * Set up, enter, and exit event loop. Blocks until event loop returns.
  */
 static void event_loop() {
-    guint source_redraw = g_idle_add(redraw, NULL);
-    guint source_timeout = g_timeout_add(100, do_timeout, NULL);
-
 #ifdef WIN32
     g_timeout_add(250, (GtkFunction) do_scriptout, NULL);
 #endif
 
     GSource *net_source = client_get_source();
-    g_assert_nonnull(net_source);
+    if (net_source == NULL) {
+        error_dialog("Server unexpectedly disconnected",
+                     "The server unexpectedly disconnected.");
+        client_disconnect();
+        return;
+    }
     g_source_set_callback(net_source, (GSourceFunc)do_network, NULL, NULL);
     g_source_attach(net_source, NULL);
     gtk_main();
 
-    g_source_remove(source_redraw);
-    g_source_remove(source_timeout);
     LOG(LOG_DEBUG, "event_loop", "Disconnected");
 }
 
@@ -332,6 +319,7 @@ static void init_ui() {
         g_error_free(error);
         exit(EXIT_FAILURE);
     }
+    LOG(LOG_DEBUG, "init_ui", "loaded dialog_xml");
 
     /* Load main window using GtkBuilder. */
     window_xml = gtk_builder_new();
@@ -341,6 +329,15 @@ static void init_ui() {
             g_error("Could not load default layout!");
         }
     }
+    LOG(LOG_DEBUG, "init_ui", "loaded window_xml");
+
+    connect_window = GTK_WIDGET(gtk_builder_get_object(dialog_xml, "connect_window"));
+    gtk_window_set_transient_for(GTK_WINDOW(connect_window),
+                                 GTK_WINDOW(window_root));
+    g_signal_connect(connect_window, "destroy",
+                     G_CALLBACK(on_window_destroy_event), NULL);
+    main_notebook =
+        GTK_NOTEBOOK(gtk_builder_get_object(dialog_xml, "main_notebook"));
 
     /* Begin connecting signals for the root window. */
     window_root = GTK_WIDGET(gtk_builder_get_object(window_xml, "window_root"));
@@ -386,6 +383,7 @@ static void init_ui() {
         }
     }
 
+    LOG(LOG_DEBUG, "init_ui", "sub init");
     inventory_init(window_root);
     info_init(window_root);
     keys_init(window_root);
@@ -396,10 +394,14 @@ static void init_ui() {
     init_create_character_window();
     metaserver_ui_init();
 
+    LOG(LOG_DEBUG, "init_ui", "window positions");
     load_window_positions(window_root);
 
+    LOG(LOG_DEBUG, "init_ui", "init themes");
     init_theme();
+    LOG(LOG_DEBUG, "init_ui", "load themes");
     load_theme(TRUE);
+    LOG(LOG_DEBUG, "init_ui", "menu items");
     init_menu_items();
 }
 
@@ -408,6 +410,7 @@ static void init_ui() {
  * new loginmethod, or after character is selected.
  */
 void show_main_client() {
+    hide_all_login_windows();
     gtk_widget_show(window_root);
     clear_stat_mapping();
     map_init(window_root);
@@ -429,12 +432,14 @@ void hide_main_client() {
     if (server != NULL) {
         sound_server_stop(server);
     }
+    gtk_widget_show(connect_window);
 }
 
 /**
  * Main client entry point.
  */
 int main(int argc, char *argv[]) {
+    global_time = g_timer_new();
 #ifdef ENABLE_NLS
     bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
     bindtextdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
@@ -455,9 +460,12 @@ int main(int argc, char *argv[]) {
     g_free(layout);
 
     // Initialize UI, sockets, and sound server.
+    LOG(LOG_DEBUG, "main", "init UI");
     init_ui();
+    LOG(LOG_DEBUG, "main", "init sockets");
     init_sockets();
 
+    LOG(LOG_DEBUG, "main", "init sound");
     if (!want_config[CONFIG_SOUND] || !init_sounds()) {
         use_config[CONFIG_SOUND] = FALSE;
     } else {
@@ -465,13 +473,31 @@ int main(int argc, char *argv[]) {
     }
 
     /* Load cached pixmaps. */
+    LOG(LOG_DEBUG, "main", "init image cache");
     init_image_cache_data();
 
+    LOG(LOG_DEBUG, "main", "init done");
+
     while (true) {
-        metaserver_show_prompt();
-        gtk_main();
+        gtk_widget_show(connect_window);
+        if (connect_server == NULL) {
+            metaserver_show_prompt();
+            gtk_main();
+        } else {
+            client_connect(connect_server);
+            if (csocket.fd == NULL) {
+                LOG(LOG_ERROR, "main", "Unable to connect to %s!", connect_server);
+                break;
+            }
+            cpl.input_state = Playing;
+        }
 
         client_negotiate(use_config[CONFIG_SOUND]);
+        if (serverloginmethod) {
+            account_show_login();
+        } else {
+            show_main_client();
+        }
 
         /* The event_loop will block until connection to the server is lost. */
         event_loop();
